<?php

define('SECURE_PAY_DB_PREFIX', 'payment_securepayapi_');

class ControllerExtensionPaymentSecurePayApi extends SecurePayApiController
{
    public function __construct($registry)
    {
        parent::__construct($registry);
        $this->load->model('sale/order');
    }

    public function index()
    {
        $this->load->language('extension/payment/securepayapi');
        $this->document->setTitle($this->language->get('heading_title'));
        $this->load->model('setting/setting');

        if ($this->validate($data)) {
            $this->model_setting_setting->editSetting('payment_securepayapi', $this->request->post);
            $data['save_success'] = true;
        }

        $data['breadcrumbs'] = array(
            array(
                'text' => $this->language->get('text_home'),
                'href' => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'], true),
                'separator' => false
            ),
            array(
                'text' => $this->language->get('text_extension'),
                'href' => $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true)
            ),
            array(
                'text' => $this->language->get('heading_title'),
                'href' => $this->url->link('extension/payment/securepayapi', 'user_token=' . $this->session->data['user_token'], true),
                'separator' => ' :: '
            )
        );

        $data['action'] = $this->url->link('extension/payment/securepayapi', 'user_token=' . $this->session->data['user_token'], true);
        $data['cancel'] = $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true);

        $data['header'] = $this->load->controller('common/header');
        $data['column_left'] = $this->load->controller('common/column_left');
        $data['footer'] = $this->load->controller('common/footer');

        $this->response->setOutput($this->load->view('extension/payment/securepayapi', $data));
    }

    private function validate(&$data)
    {
        $valid = false;
        $names = $this->uniquify_array(['tittle', 'status', 'sort_order', 'capture', 'allowed_card_types', 'store_uid', 'merchant_code', 'client_id', 'client_secret', 'test_mode']);
        $required_names = $this->uniquify_array(['tittle', 'store_uid', 'merchant_code', 'client_id', 'client_secret']);

        foreach ($names as $name) {
            $data[$name] = $this->config->get($name);
        }

        foreach ($required_names as $name) {
            $error_key = 'error_' . str_replace(SECURE_PAY_DB_PREFIX, '', $name);
            $data[$error_key] = false;
        }

        if (!$this->user->hasPermission('modify', 'extension/payment/securepayapi')) {
            $data['warning'] = $this->language->get('error_permission');
            return $valid;
        }

        if ($this->request->server['REQUEST_METHOD'] == 'POST') {
            $valid = true;
            foreach ($names as $name) {
                $data[$name] = $this->request->post[$name];
                {
                    if (in_array($name, $required_names) && strlen($this->request->post[$name]) == 0) {
                        $error_key = 'error_' . str_replace(SECURE_PAY_DB_PREFIX, '', $name);
                        $data[$error_key] = $this->language->get($error_key);
                        $valid = false;
                    }
                }
            }
        }

        return $valid;
    }

    private function uniquify_array($strs = array())
    {
        $ret = array();
        foreach ($strs as $str) {
            $ret[] = SECURE_PAY_DB_PREFIX . $str;
        }
        return $ret;
    }

    public function refund()
    {
        if (($this->request->server['REQUEST_METHOD'] == 'POST')) {
            $orderId = $this->request->post['order_id'];
            $amount = $this->request->post['amount'];
            $order = $this->model_sale_order->getOrder($orderId);
            $scope = array(SecurePayApi::SCOPE_PAYMENT_WRITE);
            $securePayApi = new SecurePayApi($this->storeId);

            $init_result = $securePayApi->initWithAuthentication($this->clientId,
                $this->clientSecret,
                $this->merchantCode,
                $scope,
                $this->isLive);
            if (true === $init_result['error']) {
                echo json_encode(
                    array(
                        'success' => false,
                        'message' => $this->general_error_message
                    )
                );
            } else {
                $refund_result = $securePayApi->refundPayment($order['ip'], $amount * 100, $orderId);
                if ($refund_result['error']) {
                    echo json_encode(
                        array(
                            'success' => false,
                            'message' => isset($refund_result['gatewayResponseMessage']) ? $refund_result['gatewayResponseMessage'] : $refund_result['errorDetail']
                        )
                    );
                } else {
                    $left = $this->currency->getSymbolLeft($this->session->data['currency']);
                    $right = $this->currency->getSymbolRight($this->session->data['currency']);
                    echo json_encode(
                        array(
                            'success' => true,
                            'comment' => sprintf($this->language->get('note_refund'), $left . $amount . $right, $refund_result['bankTransactionId'])
                        )
                    );
                }
            }
        }
    }

    public function capture()
    {
        if (($this->request->server['REQUEST_METHOD'] == 'POST')) {
            $orderId = $this->request->post['order_id'];
            $order = $this->model_sale_order->getOrder($orderId);
            $scope = array(SecurePayApi::SCOPE_PAYMENT_WRITE);
            $securePayApi = new SecurePayApi($this->storeId);

            $init_result = $securePayApi->initWithAuthentication($this->clientId,
                $this->clientSecret,
                $this->merchantCode,
                $scope,
                $this->isLive);
            if (false === $init_result['error']) {
                $capture_result = $securePayApi->capturePreAuthPayment($order['ip'], $orderId, $order['total'] * 100);
                if (!$capture_result['error']) {
                    echo json_encode(
                        array(
                            'success' => true,
                            'comment' => sprintf($this->language->get('note_capture'), $capture_result['bankTransactionId'])
                        )
                    );
                } else {
                    echo json_encode(
                        array(
                            'success' => false,
                            'message' => $this->general_error_message . "\n" . $capture_result['errorDetail']
                        )
                    );
                }
            } else {
                echo json_encode(
                    array(
                        'success' => false,
                        'message' => $this->general_error_message
                    )
                );
            }
        }
    }

    private function uniquify($str = '')
    {
        return SECURE_PAY_DB_PREFIX . $str;
    }
}
