<?php

class ControllerExtensionPaymentSecurePayApi extends SecurePayApiController
{
    public function __construct($registry)
    {
        parent::__construct($registry);
        $this->load->model('checkout/order');
    }

    public function index()
    {
        $data['payment_code'] = 'securepayapi';
        $data['order_id'] = $this->session->data['order_id'];

        $allowedCardTypes = array();

        foreach ($this->config->get('payment_securepayapi_allowed_card_types') as $cardType => $enabled) {
            if ($enabled == 1) {
                $allowedCardTypes[] = $cardType;
            }
        }

        $data['allowed_card_types'] = json_encode($allowedCardTypes);
        $data['client_id'] = $this->config->get('payment_securepayapi_client_id');
        $data['merchant_code'] = $this->config->get('payment_securepayapi_merchant_code');
        $data['securepayapi_script_url'] = $this->config->get('payment_securepayapi_test_mode') ? SecurePayApi::URL_SANDBOX_SCRIPT : SecurePayApi::URL_LIVE_SCRIPT;

//        var_dump($data);

        return $this->load->view('extension/payment/securepayapi', $data);
    }

    public function confirm()
    {
        if (($this->request->server['REQUEST_METHOD'] == 'POST')) {
            $is_save_credit_card = $this->request->post['saveCard'];
            $token = $this->request->post['securePayApiToken'];
            $orderId = $this->request->post['orderId'];
            $order = $this->model_checkout_order->getOrder($orderId);

            $scope = array(SecurePayApi::SCOPE_PAYMENT_WRITE);
            if ($is_save_credit_card) {
                $scope = array_merge($scope, array(SecurePayApi::SCOPE_PAYMENT_INSTRUMENTS_WRITE));
            }
            $securePayApi = new SecurePayApi($this->storeId);
            $init_result = $securePayApi->initWithAuthentication($this->clientId,
                $this->clientSecret,
                $this->merchantCode,
                $scope,
                $this->isLive);
            if (false === $init_result['error']) {
                if ($this->isCaptureImmediately) {
                    $create_payment_result = $securePayApi->createPayment($token,
                        $order['ip'],
                        $order['total'] * 100,
                        $orderId
                    );
                } else {
                    $create_payment_result = $securePayApi->createPreAuthPayment($token,
                        $order['ip'],
                        $order['total'] * 100,
                        $orderId
                    );
                }
                if ($create_payment_result['error']) {
                    echo json_encode(
                        [
                            'success' => false,
                            'message' => isset($create_payment_result['gatewayResponseMessage']) ? $this->transaction_declined_message . $create_payment_result['gatewayResponseMessage'] : $this->general_error_message
                        ]
                    );
                } else {
                    if ($this->isCaptureImmediately) {
                        $this->model_checkout_order->addOrderHistory($orderId, self::ORDER_STATUS_PROCESSING,
                            sprintf($this->language->get('note_paid'), $create_payment_result['bankTransactionId']),
                            true
                        );
                    } else {
                        $this->model_checkout_order->addOrderHistory($orderId, self::ORDER_STATUS_PENDING,
                            sprintf($this->language->get('note_authorized'), $create_payment_result['bankTransactionId']),
                            true
                        );
                    }
                    if ($is_save_credit_card) {
                        $securePayApi->createPaymentInstrument($order['customer_id'], $token, $order['ip']);
                    }
                    echo json_encode([
                            'success' => true,
                            'redirect' => $this->url->link('checkout/success', '', 'SSL')]
                    );
                }
            } else {
                echo json_encode(
                    [
                        'success' => false,
                        'message' => isset($init_result['gatewayResponseMessage']) ? $this->transaction_declined_message . $init_result['gatewayResponseMessage'] : $this->general_error_message
                    ]
                );
            }
        }
    }
}
